Method: HaveAPI::Client::Authentication::Base#save

Defined in:
lib/haveapi/client/authentication/base.rb

#saveObject

Returns a hash of auth provider attributes to be saved e.g. in a file to be used later, without the user providing credentials again. You may wish to save a username or password (not recommended), tokens or whatever authentication provider needs to authenticate user without his input.



69
70
71
# File 'lib/haveapi/client/authentication/base.rb', line 69

def save
  @opts
end